// Copyright (c) 2013 by István Váradi

// This file is part of XPLRA, a remote-access plugin for X-Plane

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

#ifndef XPLRA_DATAREFTASK_H
#define XPLRA_DATAREFTASK_H
//------------------------------------------------------------------------------

#include "Task.h"

#include <XPLMDataAccess.h>

#include <string>

//------------------------------------------------------------------------------

namespace xplra {

//------------------------------------------------------------------------------

/**
 * A task that works with some dataref. This base class
 * performs the mapping from dataref name to dataref value. The actual
 * operation is performed by the subclasses.
 */
class DataRefTask : public Task
{
private:
    /**
     * The name of the dataref.
     */
    std::string name;

    /**
     * The dataref.
     */
    XPLMDataRef dataRef;

protected:
    /**
     * Construct the dataref task for the dataref with the given name.
     */
    DataRefTask(const std::string& name);

    /**
     * Construct the dataref task for the given dataref.
     */
    DataRefTask(XPLMDataRef dataRef);

    /**
     * Determine if the task is valid, which is so, if the dataref is
     * not 0.
     */
    bool isValid() const;

    /**
     * Get the name of the dataref.
     */
    const std::string& getName() const;

    /**
     * Get the dataref.
     */
    XPLMDataRef getDataRef() const;

    /**
     * Perform the task. If the dataref is not valid yet, it will be
     * queried. If the query is successful, the process() function is
     * called.
     */
    virtual void perform();

    /**
     * Process the dataref. When this function is called, the dataref
     * is valid.
     */
    virtual void process() = 0;
};

//------------------------------------------------------------------------------
// Inline definitions
//------------------------------------------------------------------------------

inline DataRefTask::DataRefTask(const std::string& name) :
    name(name),
    dataRef(0)
{
}

//------------------------------------------------------------------------------

inline DataRefTask::DataRefTask(XPLMDataRef dataRef) :
    dataRef(dataRef)
{
}

//------------------------------------------------------------------------------

inline bool DataRefTask::isValid() const
{
    return dataRef!=0;
}

//------------------------------------------------------------------------------

inline const std::string& DataRefTask::getName() const
{
    return name;
}

//------------------------------------------------------------------------------

inline XPLMDataRef DataRefTask::getDataRef() const
{
    return dataRef;
}

//------------------------------------------------------------------------------

} /* namespace xplra */

//------------------------------------------------------------------------------
#endif // XPLRA_DATAREFTASK_H

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
